@@ -634,6 +634,9 @@ hb_font_t *           pango_font_get_hb_font       (PangoFont        *font);
 PANGO_AVAILABLE_IN_1_50
 PangoLanguage **      pango_font_get_languages     (PangoFont        *font);
 
+PANGO_AVAILABLE_IN_1_50
+GBytes *              pango_font_serialize         (PangoFont        *font);
+
 /**
  * PANGO_GLYPH_EMPTY:
  *
@@ -689,6 +692,7 @@ PangoLanguage **      pango_font_get_languages     (PangoFont        *font);
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(PangoFontFamily, g_object_unref)
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(PangoFontFace, g_object_unref)
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(PangoFont, g_object_unref)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(PangoFontDescription, pango_font_description_free)
 
 G_END_DECLS
 
