@@ -75,6 +75,8 @@ static gboolean             _pango_fc_font_is_hinted      (PangoFont        *fon
 static void                 _pango_fc_font_get_scale_factors (PangoFont     *font,
                                                               double        *x_scale,
                                                               double        *y_scale);
+static void                 pango_fc_font_get_matrix      (PangoFont        *font,
+                                                           PangoMatrix      *matrix);
 
 #define PANGO_FC_FONT_LOCK_FACE(font)	(PANGO_FC_FONT_GET_CLASS (font)->lock_face (font))
 #define PANGO_FC_FONT_UNLOCK_FACE(font)	(PANGO_FC_FONT_GET_CLASS (font)->unlock_face (font))
@@ -109,6 +111,7 @@ pango_fc_font_class_init (PangoFcFontClass *class)
   pclass->get_languages = _pango_fc_font_get_languages;
   pclass->is_hinted = _pango_fc_font_is_hinted;
   pclass->get_scale_factors = _pango_fc_font_get_scale_factors;
+  pclass->get_matrix = pango_fc_font_get_matrix;
 
   /**
    * PangoFcFont:pattern:
@@ -331,10 +334,11 @@ pango_fc_font_get_coverage (PangoFont     *font,
  */
 static void
 get_face_metrics (PangoFcFont      *fcfont,
-		  PangoFontMetrics *metrics)
+                  PangoFontMetrics *metrics)
 {
   hb_font_t *hb_font = pango_font_get_hb_font (PANGO_FONT (fcfont));
   hb_font_extents_t extents;
+  hb_position_t position;
 
   FcMatrix *fc_matrix;
   gboolean have_transform = FALSE;
@@ -342,10 +346,10 @@ get_face_metrics (PangoFcFont      *fcfont,
   hb_font_get_extents_for_direction (hb_font, HB_DIRECTION_LTR, &extents);
 
   if  (FcPatternGetMatrix (fcfont->font_pattern,
-			   FC_MATRIX, 0, &fc_matrix) == FcResultMatch)
+                           FC_MATRIX, 0, &fc_matrix) == FcResultMatch)
     {
       have_transform = (fc_matrix->xx != 1 || fc_matrix->xy != 0 ||
-			fc_matrix->yx != 0 || fc_matrix->yy != 1);
+                        fc_matrix->yx != 0 || fc_matrix->yy != 1);
     }
 
   if (have_transform)
@@ -366,10 +370,6 @@ get_face_metrics (PangoFcFont      *fcfont,
   metrics->strikethrough_thickness = PANGO_SCALE;
   metrics->strikethrough_position = metrics->ascent / 2;
 
-  /* FIXME: use the right hb version */
-#if HB_VERSION_ATLEAST(2,5,4)
-  hb_position_t position;
-
   if (hb_ot_metrics_get_position (hb_font, HB_OT_METRICS_TAG_UNDERLINE_SIZE, &position))
     metrics->underline_thickness = position;
 
@@ -381,7 +381,6 @@ get_face_metrics (PangoFcFont      *fcfont,
 
   if (hb_ot_metrics_get_position (hb_font, HB_OT_METRICS_TAG_STRIKEOUT_OFFSET, &position))
     metrics->strikethrough_position = position;
-#endif
 }
 
 PangoFontMetrics *
@@ -849,7 +848,7 @@ pango_fc_font_get_features (PangoFont    *font,
       char *s;
       while (*num_features < len &&
              FcResultMatch == FcPatternGetString (fc_font->font_pattern,
-                                                  PANGO_FC_FONT_FEATURES,
+                                                  FC_FONT_FEATURES,
                                                   *num_features,
                                                   (FcChar8 **) &s))
         {
@@ -881,29 +880,25 @@ pango_fc_font_key_get_gravity (PangoFcFontKey *key)
   return gravity;
 }
 
-static double
-get_font_size (PangoFcFontKey *key)
+static void
+get_font_size (PangoFcFontKey *key,
+               double         *pixel_size,
+               double         *point_size)
 {
   const FcPattern *pattern;
-  double size;
   double dpi;
 
   pattern = pango_fc_font_key_get_pattern (key);
-  if (FcPatternGetDouble (pattern, FC_PIXEL_SIZE, 0, &size) == FcResultMatch)
-    return size;
+  if (FcPatternGetDouble (pattern, FC_SIZE, 0, point_size) != FcResultMatch)
+    *point_size = 13.;
 
-  /* Just in case FC_PIXEL_SIZE got unset between pango_fc_make_pattern()
-   * and here.  That would be very weird.
-   */
+  if (FcPatternGetDouble (pattern, FC_PIXEL_SIZE, 0, pixel_size) != FcResultMatch)
+    {
+      if (FcPatternGetDouble (pattern, FC_DPI, 0, &dpi) != FcResultMatch)
+        dpi = 72.;
 
-  if (FcPatternGetDouble (pattern, FC_DPI, 0, &dpi) != FcResultMatch)
-    dpi = 72;
-
-  if (FcPatternGetDouble (pattern, FC_SIZE, 0, &size) == FcResultMatch)
-    return size * dpi / 72.;
-
-  /* Whatever */
-  return 18.;
+      *pixel_size = *point_size * dpi / 72.;
+    }
 }
 
 static void
@@ -946,10 +941,12 @@ pango_fc_font_create_hb_font (PangoFont *font)
   hb_font_t *hb_font;
   double x_scale_inv, y_scale_inv;
   double x_scale, y_scale;
-  double size;
+  double pixel_size;
+  double point_size;
 
   x_scale_inv = y_scale_inv = 1.0;
-  size = 1.0;
+  pixel_size = 1.0;
+  point_size = 1.0;
 
   key = _pango_fc_font_get_font_key (fc_font);
   if (key)
@@ -984,7 +981,7 @@ pango_fc_font_create_hb_font (PangoFont *font)
           x_scale_inv = -x_scale_inv;
           y_scale_inv = -y_scale_inv;
         }
-      size = get_font_size (key);
+      get_font_size (key, &pixel_size, &point_size);
     }
 
   x_scale = 1. / x_scale_inv;
@@ -994,8 +991,9 @@ pango_fc_font_create_hb_font (PangoFont *font)
 
   hb_font = hb_font_create (hb_face);
   hb_font_set_scale (hb_font,
-                     size * PANGO_SCALE * x_scale,
-                     size * PANGO_SCALE * y_scale);
+                     pixel_size * PANGO_SCALE * x_scale,
+                     pixel_size * PANGO_SCALE * y_scale);
+  hb_font_set_ptem (hb_font, point_size);
 
   if (key)
     {
@@ -1025,7 +1023,7 @@ pango_fc_font_create_hb_font (PangoFont *font)
           hb_ot_var_named_instance_get_design_coords (hb_face, instance, &n_axes, coords);
         }
 
-      if (FcPatternGetString (pattern, PANGO_FC_FONT_VARIATIONS, 0, (FcChar8 **)&variations) == FcResultMatch)
+      if (FcPatternGetString (pattern, FC_FONT_VARIATIONS, 0, (FcChar8 **)&variations) == FcResultMatch)
         parse_variations (variations, axes, n_axes, coords);
 
       variations = pango_fc_font_key_get_variations (key);
@@ -1115,3 +1113,22 @@ _pango_fc_font_get_scale_factors (PangoFont *font,
 
   pango_matrix_get_font_scale_factors (&fcfont->matrix, x_scale, y_scale);
 }
+
+static void
+pango_fc_font_get_matrix (PangoFont   *font,
+                          PangoMatrix *matrix)
+{
+  PangoFcFont *fcfont = PANGO_FC_FONT (font);
+  FcMatrix fc_matrix, *fc_matrix_val;
+
+  FcMatrixInit (&fc_matrix);
+  for (int i = 0; FcPatternGetMatrix (fcfont->font_pattern, FC_MATRIX, i, &fc_matrix_val) == FcResultMatch; i++)
+    FcMatrixMultiply (&fc_matrix, &fc_matrix, fc_matrix_val);
+
+  matrix->xx = fc_matrix.xx;
+  matrix->xy = - fc_matrix.xy;
+  matrix->yx = - fc_matrix.yx;
+  matrix->yy = fc_matrix.yy;
+  matrix->x0 = 0.;
+  matrix->y0 = 0.;
+}
